#!/usr/bin/env python

import os
from plugins.plugin import Plugin

# at first, define the function which imports the classes in the plugins folder
def find_subclasses(path, cls):
  """
  Find all subclass of cls in py files located below path
  (does look in sub directories)

  @param path: the path to the top level folder to walk
  @type path: str
  @param cls: the base class that all subclasses should inherit from
  @type cls: class
  @rtype: list
  @return: a list if classes that are subclasses of cls
  """

  subclasses=[]

  def look_for_subclass(modulename):
    module=__import__(modulename)

    #walk the dictionaries to get to the last one
    d=module.__dict__
    for m in modulename.split('.')[1:]:
      d=d[m].__dict__

    #look through this dictionary for things
    #that are subclass of Job
    #but are not Job itself
    for key, entry in d.items():
      if key == cls.__name__:
        continue

      try:
        if issubclass(entry, cls):
          print("Found subclass: "+key)
          subclasses.append(entry)
      except TypeError:
        #this happens when a non-type is passed in to issubclass. We
        #don't care as it can't be a subclass of Job if it isn't a
        #type
        continue

  # walk through the given path and take all the .py files
  # the plugins are loaded in alphabetical order
  for root, dirs, files in os.walk(path):
    sorted_files = []
    for filename in files:
      # collect the files in the plugins folder
      sorted_files.append(filename)
    sorted_files.sort()
    for name in sorted_files:
      if name.endswith(".py") and not name.startswith("__"):
        path = os.path.join(root, name)
        modulename = path.rsplit('.', 1)[0].replace('/', '.')
        look_for_subclass(modulename)

  return subclasses

# here is the beginning of the program
# the find_subclasses function gets the path to the plugins folder and the name of the abstract class Plugin which I imported above
imported_modules = find_subclasses("plugins", Plugin)

# lets create an instance of every found class and call the help funktion
for i in range(len(imported_modules)):
  print("Help: " + imported_modules[i]().help())